;;;; mother modular generic compiler project
; (c)2002/GPL Oskar Schirmer <oskar@scara.com>
; see file COPYING for GPL license details

; fortran 77 punchcard preparation
; according to Gehrke, "FORTRAN-77-Handbuch"

; usage:
;   punchcard 72 <original.f

; purpose:
;   eliminate comments
;   splice continuation lines

; convention:
;   lines with "C", "c" or "*" in column 1 are comment lines
;   empty lines are comment lines
;   all text beyond column 72 is comment
;   lines without "0" or " " in column 6 are continuation lines
;   column 6 is not programm text, so column 6 is a comment, too
;   no more than 19 continuation lines are allowed

(define write-error
  (lambda (y x text)
    (write
      (list 'error (cons y x) (string-append "Fortran PunchCard: " text))
      (current-error-port))
    (newline (current-error-port))))

(define write-space
  (lambda (space)
    (if (> space 0)
      (begin
        (write-char #\space)
        (write-space (- space 1))))))

(define comment-card?
  (lambda (input column)
    (and
      (= column 1)
      (assoc input '((#\C) (#\c) (#\*))))))

(define continuation-card?
  (lambda (input column)
    (and
      (= column 6)
      (not (assoc input '((#\space) (#\0)))))))

(define commentcard
  (lambda (input extra y conti)
    (if (eof-object? input)
      (write-error y 1 "unexpected end-of-file in comment card")
      (if (char=? input #\newline)
        (nextcard (read-char) 1 (+ extra 1) (+ y 1) conti)
        (begin
          (write-char #\space)
          (commentcard (read-char) extra y conti))))))

(define nextcard
  (lambda (input column extra y conti)
    (if (not (eof-object? input))
      (if (char=? input #\newline)
        (nextcard (read-char) 1 (+ extra 1) (+ y 1) conti)
        (if
          (comment-card? input column)
          (begin
            (write-char #\space)
            (commentcard (read-char) extra y conti))
          (if
            (continuation-card? input column)
            (if (>= conti 19)
              (write-error y column "too many continuation cards")
              (begin
                (write-space 6)
                (punchcard (read-char) (+ column 1) extra y (+ conti 1))))
            (if
              (and
                (< column 6)
                (char=? input #\space))
              (nextcard (read-char) (+ column 1) extra y conti)
              (if (> extra 0)
                (begin
                  (newline)
                  (nextcard input column (- extra 1) y conti))
                (begin
                  (write-space (- column 1))
                  (write-char input)
                  (punchcard (read-char) (+ column 1) 0 y 0))))))))))

(define punchcard
  (lambda (input column extra y conti)
    (if (not (eof-object? input))
      (if (char=? input #\newline)
        (nextcard (read-char) 1 (+ extra 1) (+ y 1) conti)
        (if (> column columns)
          (begin
            (write-char #\space)
            (punchcard (read-char) column extra y conti))
          (if
            (comment-card? input column)
            (begin
              (write-char #\space)
              (commentcard (read-char) extra y conti))
            (if
              (continuation-card? input column)
              (write-error y column "continuation not allowed")
              (begin
                (write-char input)
                (punchcard (read-char) (+ column 1) extra y conti)))))))))

(define columns (string->number (cadr (argv))))
(punchcard (read-char) 1 0 1 0)
(newline)
